#include "Constantes.h"
#include "Agua.h"

// Constructor
Agua::Agua(const string &name) : Atomic(name),
		Pedido_in(addInputPort("Pedido_in")),
		Agua_out(addOutputPort("Agua_out")) {
}

// Inicializacion
Model& Agua::initFunction() {
	this->Porciones_pedidas = 0;
	this->estado = Normal;

	passivate();

	return *this;
}

// Funcion de transicion externa
Model& Agua::externalFunction(const ExternalMessage& msg) {
	if (estaPasivo()) {
		if (this->estado == Normal) {
			cerr << "DEBUG: empezando a tirar agua " << msg.value() << endl;
			Porciones_pedidas = (int) msg.value();
			this->estado = AvisandoInicio;
			holdIn(0);
		}

	}

	return *this;
}


// Funcion de transicion interna
Model& Agua::internalFunction(const InternalMessage& msg) {

	switch (this->estado) {
		case AvisandoInicio: {
			this->estado = EnviandoAgua;
			VTime v(0, 0, (int) this->Porciones_pedidas, 0);
			holdIn(v);
		} break;
		case EnviandoAgua:
			this->estado = AvisandoFin;
			holdIn(0);
		break;
		case AvisandoFin:
			this->Porciones_pedidas = 0;
			this->estado = Normal;
			passivate();
		break;

		default:
		break;
	}

	return *this;
}

// Funcion de salida
Model& Agua::outputFunction(const CollectMessage& msg) {

	switch (this->estado) {
		case AvisandoInicio:
			sendOutput(msg.time(), Agua_out, (Value) INICIO_AGUA);
		break;
		case AvisandoFin:
			sendOutput(msg.time(), Agua_out, (Value) FIN_AGUA);
		break;

		default:
		break;
	}

	return *this;
}

// Funcion holdIn propia
void Agua::holdIn(const VTime& t) {
	Atomic::holdIn(AtomicState::active, t);
}

bool Agua::estaPasivo() {
	return state() == AtomicState::passive;
}
