#include "Constantes.h"
#include "Fichas.h"

// Constructor
Fichas::Fichas(const string &name) : Atomic(name),
		Ficha_in(addInputPort("Ficha_in")),
		Eject_in(addInputPort("Eject_in")),
		Comprar_in(addInputPort("Comprar_in")),
		Lista_out(addOutputPort("Lista_out")),
		Ficha_out(addOutputPort("Ficha_out")) {
}

// Inicializacion
Model& Fichas::initFunction() {
	this->estado = Normal;
	this->FichaAdentro = false;

	passivate();

	return *this;
}

// Funcion de transicion externa
Model& Fichas::externalFunction(const ExternalMessage& msg) {

	if (msg.port() == Ficha_in) {

		if (estaPasivo()) {

			if (this->FichaAdentro) {

				cerr << "DEBUG: Ficha ingresada con ficha adentro" << endl;
				this->estado = Rechazando;
				holdIn(0);

			} else {
				
				cerr << "DEBUG: Ficha ingresada @ Fichas" << endl;
				this->FichaAdentro = true;
				this->estado = AvisandoFicha;
				holdIn(0);

			}

		}

	} else
	if (msg.port() == Eject_in) {

		if (estaPasivo()) {

			if (this->FichaAdentro) {

				this->estado = AvisandoNoFicha;
				holdIn(0);

			}

		}

	} else
	if (msg.port() == Comprar_in) {

		if (estaPasivo()) {

			if ((int) msg.value() == COMPRAR) { // Comprando

				cerr << "DEBUG: Comprando @ Fichas" << endl;

				this->estado = Comprando;
			} else { // msg.value() == RECHAZAR, rechazando
				cerr << "DEBUG: Rechazo a pedido de UC @ Fichas" << endl;
				this->estado = RechazandoXPedidoInterno;
			}

			holdIn(0);

		}

	}

	return *this;
}


// Funcion de transicion interna
Model& Fichas::internalFunction(const InternalMessage& msg) {

	switch (this->estado) {
		case RechazandoXPedidoInterno:
		case Comprando:
			cerr << "DEBUG: comprando o rechazando @ Fichas" << endl;
			this->FichaAdentro = false;
			this->estado = Normal;

			passivate();
			break;
		case Rechazando:
			cerr << "DEBUG: Rechazando @ fichas" << endl;
			this->estado = Normal;
			passivate();
			break;
		case AvisandoNoFicha:
			this->estado = Rechazando;
			this->FichaAdentro = false;
			holdIn(0);
			break;
		default:
			this->estado = Normal;
			passivate();
			break;
	}

	return *this;
}

// Funcion de salida
Model& Fichas::outputFunction(const CollectMessage& msg) {

	switch (this->estado) {

		case AvisandoFicha:
			sendOutput(msg.time(), Lista_out, (Value) 1); // lista
			break;

		case AvisandoNoFicha:
			sendOutput(msg.time(), Lista_out, (Value) 0); // No Lista
			break;
		case Rechazando:
		case RechazandoXPedidoInterno:
			sendOutput(msg.time(), Ficha_out, (Value) 1);
			break;
		default:
			break;
	}

	return *this;
}

// Funcion holdIn propia
void Fichas::holdIn(const VTime& t) {
	Atomic::holdIn(AtomicState::active, t);
}

bool Fichas::estaPasivo() {
	return state() == AtomicState::passive;
}
