#ifndef __UC_H
#define __UC_H

#include "atomic.h"
#include "Receta.h"

const int DEMORA = 1; // 3 segundos

class UC : public Atomic {
	public:
		//Constructor
		UC(const string &name = "UC");
		// Estados
		enum Estado {Normal, Recargando, IniciandoCiclo,
		             RecibiendoPedido, ConfirmandoPedido,
		             RechazandoPedido, EsperandoNotifVaso,
		             EsperandoNotifCafe, EsperandoNotifLeche,
		             EsperandoNotifChocolate, SirviendoCafe,
		             SirviendoLeche, SirviendoChocolate, CargandoVaso,
		             CargandoLeche, CargandoCafe, CargandoChocolate,
		             EntregandoPedido};
	
	protected:
		//funciones de transicion externa, interna y de output y de inicializacion
		Model& initFunction();
 		Model& externalFunction(const ExternalMessage&); // externa
		Model& internalFunction(const InternalMessage&); // interna
		Model& outputFunction(const CollectMessage&); // salida
		void holdIn(const VTime&); // holdIn sin la fase, ya que es siempre activa

		bool estaPasivo(); // Retorna true si el modelo esta pasivado

	private:
		//puertos de entrada
		const Port &Recarga;
		const Port &RecibePedido_in;
		const Port &FichaLista_in;
		const Port &Notif_in;

		//puertos de salida
		Port &Habilita_out;
		Port &Confirma_out;
		Port &PedidoCafe_out;
		Port &PedidoChoc_out;
		Port &PedidoLeche_out;
		Port &PedidoAgua_out;
		Port &PedidoVaso_out;
		Port &EnviarMensaje_out;
		
		//variables de estado
		Estado estado;
		int CantLeche;
		int CantChocolate;
		int CantCafe;
		int CantVasos;
		Receta RecetaLagrima;
		Receta RecetaCafe;
		Receta RecetaCafeConLeche;
		Receta RecetaChocolate;
		Receta sabor;
		int Mensaje;
};


#endif __UC_H
